@@ -53,6 +53,18 @@ sub time {
     return time - shift->{_time};
 }
 
+sub DESTROY {
+    my $self = shift;
+    
+    $self->{_parent}->handler(Net::IRC::Event->new('dcc_close',
+						   $self->{_nick},
+						   $self->{_socket},
+						   $self->{_type}));
+}
+
+sub peer {
+    return ( $_[0]->{_nick}, "DCC " . $_[0]->{_type} );
+}
 
 # -- #perl was here! --
 #     orev: hehe...
@@ -71,7 +83,7 @@ use IO::Socket;
 
 sub new {
 
-    my ($class, $container, $address, $port, $size, $filename) = @_;
+    my ($class, $container, $nick, $address, $port, $size, $filename) = @_;
     
     my ($sock, $fh);
 
@@ -94,25 +106,34 @@ sub new {
     $sock = new IO::Socket::INET( Proto    => "tcp",
 				  PeerAddr => "$address:$port" );
 
-    unless(defined $sock) {
+    if (defined $sock) {
+	$container->handler(Net::IRC::Event->new('dcc_open',
+						 $nick,
+						 $sock,
+						 'get',
+						 'get', $sock));
+	
+    } else {
         $container->printerr("Can't connect to $address: $!");
         $fh->close();
         return undef;
     }
-
+    
     $sock->autoflush(1);
 
     my $self = {
         _bin        =>  0,      # Bytes we've recieved thus far
         _bout       =>  0,      # Bytes we've sent
+        _connected  =>  1,
         _fh         =>  $fh,    # FileHandle we will be writing to.
         _filename   =>  $filename,
+	_nick       =>  $nick,  # Nick of person on other end
+        _parent     =>  $container,
         _size       =>  $size,  # Expected size of file
         _socket     =>  $sock,  # Socket we're reading from
         _time       =>  time, 
-        _connected  =>  1,
-        _parent     =>  $container
-    };
+	_type       =>  'get',
+        };
 
     bless $self, $class;
 
@@ -124,7 +145,7 @@ sub new {
 #  \merlyn: I mean carle
 #  \merlyn: carly
 # Silmaril: man merlyn
-# Silmaril: you shouold have offered HER the shower.
+# Silmaril: you should have offered HER the shower.
 #   \petey: all three of them?
 
 sub parse {
@@ -134,7 +155,7 @@ sub parse {
         $self->{_parent}->printerr("Error writing to " . 
              $self->{_filename} . ": $!");
         $self->{_fh}->close;
-        $self->{_parent}->parent->remove_conn($self);
+        $self->{_parent}->parent->removeconn($self);
         return;
     }
 
@@ -144,7 +165,7 @@ sub parse {
     unless ( $self->{_socket}->send( pack("N", $self->{_bin}) ) ) {
         $self->{_parent}->printerr("Error writing to socket: $!");
         $self->{_fh}->close;
-        $self->{_parent}->parent->remove_conn($self);
+        $self->{_parent}->parent->removeconn($self);
         return;
     }
 
@@ -153,7 +174,7 @@ sub parse {
     # If we close the socket, the select loop gets screwy because
     # it won't remove its reference to the socket.  weird.
     if ( $self->{_size} && $self->{_size} == $self->{_bin} ) {
-        $self->{_parent}->parent->remove_conn($self);
+        $self->{_parent}->parent->removeconn($self);
         $self->{_fh}->close;
     }
 
@@ -204,47 +225,47 @@ sub new {
 				  LocalPort => $port,
                                   Listen    => 1);
 
-    unless (defined $sock) {
+    if (defined $sock) {
+	$container->handler(Net::IRC::Event->new('dcc_open',
+						 $nick,
+						 $sock,
+						 'send',
+						 'send', $sock));
+	
+    } else {
         $container->printerr("Couldn't open socket: $!");
         $fh->close;
         return undef;
-    }
+    }    
 
     $container->ctcp('DCC SEND', $nick, $filename, 
                      unpack("N",inet_aton(hostname())), $port, $size);
 
-    # this accept() blocks.  if they don't connect, the client is hung
-    # what's the fix?
-    $sock = $sock->accept;
-
-    unless (defined $sock) {
-        $container->printerr("Error in accept: $!");
-        $fh->close;
-        return undef;
-    }
-
     $sock->autoflush(1);
 
-    # ok, to get the ball rolling, we send them the first packet.
-    # the rest will get sent from parse()
-    my $buf;
-    return undef unless defined $fh->read($buf,$blocksize);
-    return undef unless defined $sock->send($buf);
-
-
     my $self = {
         _bin        =>  0,         # Bytes we've recieved thus far
         _blocksize  =>  $blocksize,       
         _bout       =>  0,         # Bytes we've sent
         _fh         =>  $fh,       # FileHandle we will be reading from.
         _filename   =>  $filename,
+	_nick       =>  $nick,
         _parent     =>  $container,
         _size       =>  $size,     # Size of file
         _socket     =>  $sock,     # Socket we're writing to
         _time       =>  time, 
+	_type       =>  'send',
     };
 
     bless $self, $class;
+    
+    $sock = Net::IRC::DCC::Accept->new($sock, $self);
+
+    unless (defined $sock) {
+        $container->printerr("Error in accept: $!");
+        $fh->close;
+        return undef;
+    }
 
     return $self;
 }
@@ -272,7 +293,7 @@ sub parse {
     if ($size == $self->{_size}) {
         # they've acknowledged the whole file,  we outtie
         $self->{_fh}->close;
-        $self->{_parent}->parent->remove_conn($self);
+        $self->{_parent}->parent->removeconn($self);
         return;
     } 
 
@@ -282,13 +303,13 @@ sub parse {
 
     unless (defined $self->{_fh}->read($buf,$self->{_blocksize})) {
         $self->{_fh}->close;
-        $self->{_parent}->parent->remove_conn($self);
+        $self->{_parent}->parent->removeconn($self);
         return;
     }
 
     unless($self->{_socket}->send($buf)) {
         $self->{_fh}->close;
-        $self->{_parent}->parent->remove_conn($self);
+        $self->{_parent}->parent->removeconn($self);
     }
 
     $self->{_bout} += length($buf);
@@ -316,8 +337,7 @@ use Sys::Hostname;
 sub new {
 
     my ($class, $container, $type, $nick, $address, $port) = @_;
-
-    my $sock;
+    my ($sock, $self);
 
     if ($type) {
         # we're initiating
@@ -326,7 +346,8 @@ sub new {
         $sock = new IO::Socket::INET( Proto     => "tcp",
 				      LocalPort => $port,
                                       Listen    => 1);
-
+	$sock->autoflush(1);
+	
         unless (defined $sock) {
             $container->printerr("Couldn't open socket: $!");
             print("Couldn't open socket: $!");
@@ -336,40 +357,65 @@ sub new {
         $container->ctcp('DCC CHAT', $nick, 'chat',  
                          unpack("N",inet_aton(hostname)), $port);
 
-        # this will block, leaving the client in a bad position
-        $sock = $sock->accept;
-
-        unless (defined $sock) {
-            $container->printerr("Error in connect: $!");
-            return undef;
-        }
+	$self = {
+	    _bin        =>  0,      # Bytes we've recieved thus far
+	    _bout       =>  0,      # Bytes we've sent
+	    _connected  =>  1,
+	    _nick       =>  $nick,  # Nick of the client on the other end
+	    _parent     =>  $container,
+	    _socket     =>  $sock,  # Socket we're reading from
+	    _time       =>  time,
+	    _type       =>  'chat',
+	};
+	
+	bless $self, $class;
+	
+        $sock = Net::IRC::DCC::Accept->new($sock, $self);
 
+	if (defined $sock) {
+	    $container->handler(Net::IRC::Event->new('dcc_open',
+						     $nick,
+						     $sock->socket,
+						     'chat',
+						     'chat', $sock->socket));
+	    
+	} else {
+	    $container->printerr("Error in DCC CHAT connect: $!");
+	    return undef;
+	}
+	
     } else {      # we're connecting
-
+	
         $address = inet_ntoa(pack("N",$address));
         $sock = new IO::Socket::INET( Proto    => "tcp",
 				      PeerAddr => "$address:$port");
+	$sock->autoflush(1);
 
-        unless (defined $sock) {
-            $container->printerr("Error in connect: $!");
-            return undef;
-        }
+        if (defined $sock) {
+	    my $ev = Net::IRC::Event->new('dcc_open',
+					  $nick,
+					  $sock,
+					  'chat',
+					  'chat', $sock);
+	} else {
+	    $container->printerr("Error in connect: $!");
+	    return undef;
+	}
+
+	$self = {
+	    _bin        =>  0,      # Bytes we've recieved thus far
+	    _bout       =>  0,      # Bytes we've sent
+	    _connected  =>  1,
+	    _nick       =>  $nick,  # Nick of the client on the other end
+	    _parent     =>  $container,
+	    _socket     =>  $sock,  # Socket we're reading from
+	    _time       =>  time,
+	    _type       =>  'chat',
+	};
+	
+	bless $self, $class;
     }
 
-    $sock->autoflush(1);
-
-    my $self = {
-        _bin        =>  0,      # Bytes we've recieved thus far
-        _bout       =>  0,      # Bytes we've sent
-        _socket     =>  $sock,  # Socket we're reading from
-        _time       =>  time,
-	_nick       =>  $nick,  # Nick of the client on the other end
-        _connected  =>  1,
-        _parent     =>  $container
-    };
-
-    bless $self, $class;
-
     return $self;
 }
 
@@ -408,4 +454,130 @@ sub parse {
 #  \merlyn: good topic
 
 
+# Sockets waiting for accept() use this to shoehorn into the select loop.
+package Net::IRC::DCC::Accept;
+
+@Net::IRC::DCC::Accept::ISA = qw(Net::IRC::DCC::Connection);
+
+sub new {
+    my ($class, $sock, $parent) = @_;
+    my ($self);
+
+    $self = {
+	     _nonblock =>  1,
+	     _socket   =>  $sock,
+	     _parent   =>  $parent,
+	     _type     =>  'accept',
+            };
+    
+    bless $self, $class;
+
+    # Tkil's gonna love this one. :-)   But what the hell... it's safe to
+    # assume that the only thing initiating DCCs will be Connections, right?
+    $self->{_parent}->{_parent}->parent->addconn($self);
+    return $self;
+}
+
+sub parse {
+    my ($self) = shift;
+    my ($sock);
+    
+    $sock = $self->{_socket}->accept;
+    $self->{_parent}->{_socket} = $sock;
+
+    if ($self->{_parent}->{_type} eq 'send') {
+	# ok, to get the ball rolling, we send them the first packet.
+	my $buf;
+	unless (defined $self->{_parent}->{_fh}->
+		read($buf, $self->{_parent}->{_blocksize})) {
+	    return undef;
+	}
+	return undef unless defined $sock->send($buf);
+    }
+    
+    $self->{_parent}->{_parent}->parent->addconn($self->{_parent});
+    $self->{_parent}->{_parent}->parent->removeconn($self);
+}
+
+
+
 1;
+
+
+__END__
+
+=head1 NAME
+
+Net::IRC::DCC - Object-oriented interface to a single DCC connection
+
+=head1 SYNOPSIS
+
+Hard hat area: This section under construction. Watch for falling referents.
+
+=head1 DESCRIPTION
+
+This documentation is a subset of the main Net::IRC documentation. If
+you haven't already, please "perldoc Net::IRC" before continuing.
+
+Net::IRC::DCC defines a few subclasses that handle DCC CHAT, GET, and SEND
+requests for inter-client communication. DCC objects are created by
+C<Connection-E<gt>new_{chat,get,send}()> in much the same way that
+C<IRC-E<gt>newconn()> creates a new connection object.
+
+B<NOTE:> DCC CHAT and SEND currently block on the accept() call when setting up
+a new connection. This will probably change in the near future, but bear it in
+mind for the time being if you expect to use these heavily.
+
+=head1 METHOD DESCRIPTIONS
+
+This section is under construction, but hopefully will be finally written up
+by the next release. Please see the C<irctest> script and the source for
+details about this module.
+
+=head1 AUTHORS
+
+Conceived and initially developed by Greg Bacon (gbacon@adtran.com) and
+Dennis Taylor (corbeau@execpc.com).
+
+Ideas and large amounts of code donated by Nat "King" Torkington (gnat@frii.com).
+
+Currently being hacked on, hacked up, and worked over by the members of the
+Net::IRC developers mailing list. For details, see
+http://www.execpc.com/~corbeau/irc/list.html .
+
+=head1 URL
+
+The following identical pages contain up-to-date source and information about
+the Net::IRC project:
+
+=over
+
+=item *
+
+http://www.execpc.com/~corbeau/irc/
+
+=item *
+
+http://betterbox.net/fimmtiu/irc/
+
+=back
+
+=head1 SEE ALSO
+
+=over
+
+=item *
+
+perl(1).
+
+=item *
+
+RFC 1459: The Internet Relay Chat Protocol
+
+=item *
+
+http://www.irchelp.org/, home of fine IRC resources.
+
+=back
+
+=cut
